title: Nesting Components
url: http://simurai.com/blog/2015/05/11/nesting-components/
hash_url: cf86bb38cba70b66e0d7badbdae4ee09

<p>..or the struggles with contextual styling. — Using CSS components is somewhat straightforward. We add the markup and give it the component’s class name and all is good. Where it gets trickier is when we try to nest components. And when they need to be tweaked based on the context. Where should the styles be defined? It’s a question I’ve been asking myself a few times and what this article is trying to explore.</p>

<blockquote>
<p>Just to clarify before we start, with "CSS components", I mean the small building blocks that get used to assemble a website or app. Like buttons, inputs, navs, headers etc. Some also call them modules or patterns. Also I'm using the <a href="https://github.com/suitcss/suit/blob/master/doc/naming-conventions.md">SUIT</a> naming convention in the examples below, but any other convention would be fine as well. And just a heads, there isn't some awesome solution at the end that solves all the problems. It's just me whining most of the time.</p>
</blockquote>

<p>Ok, best is to go straight into it and look at an example. Let’s say we have a Header component where we would like to add a Button component inside.</p>
<div class="highlight"><pre><code class="language-html" data-lang="html"><span class="nt">&lt;header</span> <span class="na">class=</span><span class="s">“Header”</span><span class="nt">&gt;</span>
  <span class="nt">&lt;button</span> <span class="na">class=</span><span class="s">“Button”</span><span class="nt">&gt;</span>Button<span class="nt">&lt;/button&gt;</span>
<span class="nt">&lt;/header&gt;</span>
</code></pre></div>
<p>Now because the Button is inside the Header, we want to make the Button a bit smaller than it would be on its own.</p>

<p><img src="/img/posts/nesting-components-1.png" alt="Button in Header"/></p>

<p>Here a few approaches how to do that:</p>

<h2 id="option-1---descendant-selector">Option 1 - Descendant selector</h2>

<p>Maybe the most common way is to use a descendant selector to change the <code>font-size</code> whenever a Button is inside a Header.</p>
<div class="highlight"><pre><code class="language-css" data-lang="css"><span class="nc">.Header</span> <span class="nc">.Button</span> <span class="p">{</span>
  <span class="k">font-size</span><span class="o">:</span> <span class="m">.75em</span><span class="p">;</span>
<span class="p">}</span>
</code></pre></div>
<p>This works great but the question is, where should this rule be added? We probably split our components into separate files, so is it in <code>header.scss</code> or in <code>button.scss</code>? In other words, should the Header know about what other components might get nested or should the Button know in what environment it will get placed?</p>

<p>But wait, the point of creating components is to separate them, make them modular. Each component should be kept isolated and shouldn’t know about other components. So we can make changes, rename or remove them without having to check if they might get used somewhere else.</p>

<h2 id="option-2---variations">Option 2 - Variations</h2>

<p>Another way is to create variations. We add a <code>.Button--small</code> class that we can use whenever we would like the button to be smaller without having to worry about ancestors.</p>
<div class="highlight"><pre><code class="language-css" data-lang="css"><span class="nc">.Button--small</span> <span class="p">{</span>
  <span class="k">font-size</span><span class="o">:</span> <span class="m">.75em</span><span class="p">;</span>
<span class="p">}</span>
</code></pre></div><div class="highlight"><pre><code class="language-html" data-lang="html"><span class="nt">&lt;header</span> <span class="na">class=</span><span class="s">“Header”</span><span class="nt">&gt;</span>
  <span class="nt">&lt;button</span> <span class="na">class=</span><span class="s">“Button</span> <span class="na">Button--small</span><span class="err">”</span><span class="nt">&gt;</span>Button<span class="nt">&lt;/button&gt;</span>
<span class="nt">&lt;/header&gt;</span>
</code></pre></div>
<p>This works great too, but could get out of hand quickly. What do you do if at some point you want the <code>font-size</code> to be <code>.9em</code>? Create yet another variation? <code>Button--justALittleSmaller</code>. As the project keeps growing, the number of variations will too. We  will start to loose sight where they actually get used and we’re not sure anymore if we can change a variation or if it will have <a href="http://philipwalton.com/articles/side-effects-in-css/">side effects</a> in some other place. We could create “contextual” variations like <code>Button--header</code> or <code>Button--footer</code>, but then we’re back at the beginning and could just as well use “descendant selectors”.</p>

<p>Same goes for using states. <code>.Button.is-small</code> should only be used if there is a change in state and not to fit a certain context.</p>

<h2 id="option-3---adopted-child">Option 3 - Adopted Child</h2>

<p>I can’t remember where I read about this approach but somehow it stuck with me. I also forgot how it was called. So for now I’ll just call it “Adopted Child”.</p>

<p>Let’s switch it around and look at it from the Header’s perspective. What would we do if we wouldn’t know what the components are called that might get nested? But we know that we want to make them a bit smaller. Well, we probably would create  a generic <code>.Header-item</code> class and use it like this:</p>
<div class="highlight"><pre><code class="language-css" data-lang="css"><span class="nc">.Header-item</span> <span class="p">{</span>
  <span class="k">font-size</span><span class="o">:</span> <span class="m">.75em</span><span class="p">;</span>
<span class="p">}</span>
</code></pre></div><div class="highlight"><pre><code class="language-html" data-lang="html"><span class="nt">&lt;header</span> <span class="na">class=</span><span class="s">“Header”</span><span class="nt">&gt;</span>
  <span class="nt">&lt;div</span> <span class="na">class=</span><span class="s">“Header-item”</span><span class="nt">&gt;&lt;/div&gt;</span>
<span class="nt">&lt;/header&gt;</span>
</code></pre></div>
<p>Ok, that gets us a bit closer. Now, it’s probably strange saying it like that when talking about CSS, but what would we do if we don’t want to create an own child, but still have one. Right, we could adopt one. In our example we adopt a Button component as our own child. We didn’t create it, but now we can tweak.. erm.. I mean “raise” it like it’s our own:</p>
<div class="highlight"><pre><code class="language-scss" data-lang="scss"><span class="c1">// born in button.scss</span>
<span class="nc">.Button</span> <span class="p">{</span>
  <span class="na">font-size</span><span class="o">:</span> <span class="mi">1</span><span class="kt">em</span><span class="p">;</span>
<span class="p">}</span>

<span class="c1">// raised in header.css</span>
<span class="nc">.Header</span> <span class="nc">.Header-item</span> <span class="p">{</span>
  <span class="na">font-size</span><span class="o">:</span> <span class="mf">.75</span><span class="kt">em</span><span class="p">;</span>
<span class="p">}</span>
</code></pre></div><div class="highlight"><pre><code class="language-html" data-lang="html"><span class="nt">&lt;header</span> <span class="na">class=</span><span class="s">“Header”</span><span class="nt">&gt;</span>
  <span class="nt">&lt;button</span> <span class="na">class=</span><span class="s">“Header-item</span> <span class="na">Button</span><span class="err">”</span><span class="nt">&gt;</span>Button<span class="nt">&lt;/button&gt;</span>
<span class="nt">&lt;/header&gt;</span>
</code></pre></div>
<p>It is a bit uncommon that the same HTML element shares classes from two different components. And it’s not without any risks. More about them later. But I really like this approach because it keeps the components independent without having to know about each other.</p>

<p>Another nice thing is that if we want to add other components to the Header that also need the same adjustments, we can reuse the same <code>Header-item</code> class, like for example on a text Input.</p>
<div class="highlight"><pre><code class="language-html" data-lang="html"><span class="nt">&lt;header</span> <span class="na">class=</span><span class="s">“Header”</span><span class="nt">&gt;</span>
    <span class="nt">&lt;input</span> <span class="na">class=</span><span class="s">“Header-item</span> <span class="na">Input</span><span class="err">”</span><span class="nt">&gt;</span>
  <span class="nt">&lt;button</span> <span class="na">class=</span><span class="s">“Header-item</span> <span class="na">Button</span><span class="err">”</span><span class="nt">&gt;</span>Button<span class="nt">&lt;/button&gt;</span>
<span class="nt">&lt;/header&gt;</span>
</code></pre></div>
<p><img src="/img/posts/nesting-components-2.png" alt="Button and Input in Header"/></p>

<p>Ok, about those risks. Well, depending on what properties we wanna change, it might not always be ideal. For example, because the Button already had <code>font-size</code> defined, we had to increase specificity by using <code>.Header .Header-item</code>. But that would also override variations like <code>.Button--small</code>. That might be how we want it, but there are also situations where we’d like the variation to always be “stronger”. An example would be when changing colors. When the color of Buttons should be different inside a Header, but not when its a variation, like <code>.Button—primary</code>. Yeah, we could take a look inside button.scss or our style-guide, but remember our goal.. we actually don’t want to make decisions by looking how other components are made.</p>

<p>So, as a general rule, don’t use “adopted children” for any properties that are theme related and only where you can be sure that you want to override them all the time. Like for layout/size related properties or adjusting the position.</p>

<h2 id="more-options?">More options?</h2>

<p>There are some more ways to do contextual styling that came to mind. I'll just mention them briefly for completeness, but think the 3 above are better suited.</p>

<p><strong>Option 4</strong> - We could use a preprocessor to <strong>extend</strong> an existing component. In our example it would be a clone of the Button with some tweaks added and used as a new child component <code>.Header-button</code>. Now we only rely that the Button exists in the source, but don't have to worry about other contexts. Downside is inflating our CSS output. As well as having to remember lots of new child component classes.</p>

<p><strong>Option 5</strong> - We could create a <strong>utility</strong> class like <code>.u-small</code>. It's similar to variations, but not scoped to a single component and could be used for other components as well. And for that reason it becomes very risky to ever change later.</p>

<p><strong>Option 6</strong> - And of course, we could use <strong>inline styles</strong>. But I would leave that to JavaScript only.</p>

<hr/>

<p>So after all that, which is best? I’m afraid there isn't a clear winner. It would be nice to keep it consistent with a single approach throughout the entire project, but I guess we just have to decide on a per case basis:</p>

<ol>
<li><strong>Descendant selectors</strong> if we can expect that components don’t change much. Like when using a UI Kit or library.</li>
<li><strong>Variations</strong> if it makes sense that a component has different versions that get reused anyways, and not just for a specific context.</li>
<li><strong>Adopted Child</strong> for layout, sizing, positioning or where we are sure to always want to override a property. Also for changing multiple child components at once.</li>
<li><strong>Extending</strong> when we truly want the components to be separated and don’t mind inflating the CSS output.</li>
<li><strong>Utilities</strong> for very specific things, that once the class is defined, it will never change, like clearing floats.</li>
<li><strong>Inline styles</strong> if it needs to be dynamically added with JavaScript.</li>
</ol>

<p>As said at the beginning, I haven't found a "fits all" solution and maybe the conclusion is: Try to keep contextual styling to a minimum.</p>

<h2 id="updates">Updates</h2>

<p>The "Adopted Child" approach is called "Mixes" in BEM. Here some <a href="https://en.bem.info/forum/issues/4/">more infos</a>.</p>

<hr/>

<p>SUIT also <a href="https://github.com/suitcss/suit/blob/master/doc/components.md#styling-dependencies">recommends</a> using "Adopted Child/Mixes". But also another option:</p>

<p><strong>Option 7</strong> - Adding a <strong>wrapper element</strong>. It's the <code>&lt;div class="Excerpt-wrapButton"&gt;</code> in that <a href="https://github.com/suitcss/suit/blob/master/doc/components.md#styling-dependencies">example</a>. I think it works great in most cases. But for example when using Flexbox, because it has this parent/child relationship, adding an extra wrapper in between would break it. And then you might still need to set the width of the wrapped component to 100% or so. Anyways, this is a great addition. Thanks Pablo in the comments.</p>

<hr/>

<p><strong>Option 8</strong> - <strong>Single Purpose Classes</strong>. It's where every class has only a single property. It's somewhere between utilities (Option 5) and inline styles (Option 6). <a href="http://acss.io">Atomic CSS</a> and <a href="http://tachyons.io/">Tachyons</a> use this approach. I haven't used them on a real project, but just from looking at it, the concerns are similar to the ones from utilites. If you want to change the value in a SP class, it seems unpredictable. Because in another place (where that same class is used), you might want to keep the current value. So you would have to first check if the change has any unwanted effects somewhere else.</p>
